﻿//Tutorial from https://roystan.net/articles/toon-water.html

Shader "Custom/Waves" {
    Properties {
        _ColorHighlight ("ColorHighlight", Color) = (0,0,0,1)
        _ColorLowlight ("Colorlowlight", Color) = (0,0,0,1)
        _tintAmount ("Tint Amount", Range(0,1)) = 0.5
        _HeightMin ("Height Min", Float) = -1
        _HeightMax ("Height Max", Float) = 1
        _Strength ("Strength", Range(-10,10)) = 1.0
        _Speed ("Speed", Range(-200,200)) = 100
        _SurfaceNoise("Surface Noise", 2D) = "white" {}
        _DepthMaxDistance("Depth Maximum Distance", Float) = 1
        _SurfaceNoiseCutoff("Surface Noise Cutoff", Range(0, 1)) = 0.777
        _FoamMaxDistance("Foam Maximum Distance", Float) = 0.4
        _FoamMinDistance("Foam Minimum Distance", Float) = 0.04
        _SurfaceNoiseScroll("Surface Noise Scroll Amount", Vector) = (0.03, 0.03, 0, 0)
        _SurfaceDistortion("Surface Distortion", 2D) = "white" {} 
        _SurfaceDistortionAmount("Surface Distortion Amount", Range(0, 1)) = 0.27
        _FoamColor("Foam Color", Color) = (1,1,1,1)
    }
    SubShader {
        Tags {
            "RenderType" = "transparent"
        }
        Pass {
            Cull Off
            Blend SrcAlpha OneMinusSrcAlpha
            ZWrite Off
            
            CGPROGRAM
            
                #pragma vertex vertexFunc
                #pragma fragment fragmentFunc
                
                #include "UnityCG.cginc"
                
                sampler2D _MainTex;
                fixed4 _ColorHighlight;
                fixed4 _ColorLowlight;
                float _tintAmount;
                int _Strength;
                int _Speed;
                float _FoamMaxDistance;
                float _FoamMinDistance;
                float4 _FoamColor;
                
                float4 alphaBlend(float4 top, float4 bottom) {
                    float3 color = (top.rgb * top.a) + (bottom.rgb * (1 - top.a));
                    float alpha = top.a + bottom.a * (1 - top.a);

                    return float4(color, alpha);
                }
                
                struct  vertextInput {
                    float4 vertex : POSITION;
                    float4 uv : TEXCOORD0;
                    float3 normal : NORMAL;
                };
                
                struct vertextOutput {
                    float4 pos : SV_POSITION;
                    float4 screenPosition : TEXCOORD2;
                    float2 noiseUV : TEXCOORD0;
                    float2 distortUV : TEXCOORD1;
                    float3 viewNormal : NORMAL;
                };
                
                sampler2D _SurfaceNoise;
                float4 _SurfaceNoise_ST;
                float _SurfaceNoiseCutoff;
                float4 _SurfaceNoiseScroll;
                sampler2D _SurfaceDistortion;
                float4 _SurfaceDistortion_ST;
                float _SurfaceDistortionAmount;
                
                vertextOutput vertexFunc(vertextInput v) {
                    vertextOutput o;
                    
                    o.pos = UnityObjectToClipPos(v.vertex);
                    
                    
                    //wave effect
                    float4 worldPos = mul(unity_ObjectToWorld, v.vertex);
                    float displacement = (cos(worldPos.y) + cos(worldPos.x + (_Speed * _Time)));
                    worldPos.y = worldPos.y + (displacement * (_Strength * 0.1));
                    
                    
                    o.pos = mul(UNITY_MATRIX_VP, worldPos); 
                    
                    
                    o.screenPosition = ComputeScreenPos(o.pos);
                    
                    o.noiseUV = TRANSFORM_TEX(v.uv, _SurfaceNoise);
                    o.distortUV = TRANSFORM_TEX(v.uv, _SurfaceDistortion);
                    
                    o.viewNormal = COMPUTE_VIEW_NORMAL;
                    
                    
                    
                    return o;
                }
                
                float _DepthMaxDistance;
                sampler2D _CameraDepthTexture;
                sampler2D _CameraNormalsTexture;
                
                fixed4 fragmentFunc(vertextOutput IN)  : SV_Target {
                
                    float existingDepth01 = tex2Dproj(_CameraDepthTexture, UNITY_PROJ_COORD(IN.screenPosition)).r;
                    float existingDepthLinear = LinearEyeDepth(existingDepth01);
                    float depthDifference = existingDepthLinear - IN.screenPosition.w;
                    
                    float waterDepthDifference01 = saturate(depthDifference / _DepthMaxDistance);
                    float4 waterColor = lerp(_ColorHighlight, _ColorLowlight, waterDepthDifference01);
                    
                    float2 distortSample = (tex2D(_SurfaceDistortion, IN.distortUV).xy * 2 - 1) * _SurfaceDistortionAmount;
                    float2 noiseUV = float2((IN.noiseUV.x + _Time.y * _SurfaceNoiseScroll.x) + distortSample.x, (IN.noiseUV.y + _Time.y * _SurfaceNoiseScroll.y) + distortSample.y);
                    float surfaceNoiseSample = tex2D(_SurfaceNoise, noiseUV).r;
                    
                    
                    float3 existingNormal = tex2Dproj(_CameraNormalsTexture, UNITY_PROJ_COORD(IN.screenPosition));
                    float3 normalDot = saturate(dot(existingNormal, IN.viewNormal));
                    
                    float foamDistance = lerp(_FoamMaxDistance, _FoamMinDistance, normalDot);
                    float foamDepthDifference01 = saturate(depthDifference / foamDistance);
                    
                    float surfaceNoiseCutoff = foamDepthDifference01 * _SurfaceNoiseCutoff;

                    float surfaceNoise = surfaceNoiseSample > surfaceNoiseCutoff ? 1 : 0;
                    
                    float4 surfaceNoiseColor = _FoamColor;
                    surfaceNoiseColor.a *= surfaceNoise;

                    return alphaBlend(surfaceNoiseColor, waterColor);
                
                    /*
                    float4 worldPos = mul(unity_ObjectToWorld, IN.pos);
                    float displacement = (cos(worldPos.y) + cos(worldPos.x + (_Speed * _Time)));
                    float tintValue = worldPos.y + (displacement * (_Strength * 0.1));
                    
                    fixed4 colorChange = lerp(_ColorHighlight,_ColorLowlight, worldPos.y);
                     
                    return colorChange;
                    
                    */
                }
            
            ENDCG
        }
        //CGPROGRAM
            
        //ENDCG
        //Pass {
            /*
            CGPROGRAM
                #pragma surface surf Lambert
          
                sampler2D _MainTex;
                fixed4 _ColorHighlight;
                fixed4 _ColorLowlight;
                float _HeightMin;
                float _HeightMax;
          
                struct Input {
                    float2 uv_MainTex;
                    float3 worldPos;
                };
          
                void surf (Input IN, inout SurfaceOutput o) {
                    half4 c = tex2D (_MainTex, IN.uv_MainTex);
                    float3 localPos = IN.worldPos -  mul(unity_ObjectToWorld, float4(0,0,0,1)).xyz;
                    
                    float h = (_HeightMax - localPos.y) / (_HeightMax - _HeightMin);
                    fixed4 tintColor = lerp(_ColorHighlight.rgba, _ColorLowlight.rgba, h);
                    o.Albedo = c.rgb * tintColor.rgb;
                    o.Alpha = c.a * tintColor.a;
                }
             ENDCG
             */
        //}
        
    }
}